@@ -17,7 +17,7 @@
  *   along with this program; if not, write to the Free Software
  *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- *   SVN:$Id: gui.c 124 2008-01-10 19:10:45Z olivier $
+ *   SVN:$Id: gui.c 132 2008-02-07 16:35:47Z olivier $
  */
 
 #include "gui/common.h"
@@ -190,15 +190,7 @@ gui_transpose_volumes (GtkWidget *adj, gui_t *gui)
 {
   GtkWidget *master = g_object_get_data (G_OBJECT (adj), "user-data");
   double *orig_volumes = g_object_get_data (G_OBJECT (master), "user-data");
-#ifdef USE_PHAT
-#ifdef USE_KNOB
-  double mvol = (double) phat_fan_slider_get_value (PHAT_FAN_SLIDER (master));
-#else
-  double mvol = (double) phat_knob_get_value (PHAT_KNOB (master));
-#endif
-#else
   double mvol = (double) gtk_spin_button_get_value (GTK_SPIN_BUTTON (master));
-#endif  
   int ntracks = sequence_get_tracks_num (gui->sequence);
   int i;
   for (i=0; i < ntracks; i++) 
@@ -247,9 +239,9 @@ gui_transpose_volumes_dialog (gui_t *gui, guint action, GtkWidget *widget)
   for (i = 0; i < ntracks; i++)
 #ifdef USE_PHAT  
 #ifdef USE_KNOB
-    orig_volumes[i] = phat_fan_slider_get_value (PHAT_FAN_SLIDER (gui->volume_spinners[i]));
-#else
     orig_volumes[i] = phat_knob_get_value (PHAT_KNOB (gui->volume_spinners[i]));
+#else
+    orig_volumes[i] = phat_fan_slider_get_value (PHAT_FAN_SLIDER (gui->volume_spinners[i]));
 #endif
 #else
     orig_volumes[i] = gtk_spin_button_get_value (GTK_SPIN_BUTTON (gui->volume_spinners[i]));
