@@ -1,7 +1,7 @@
 /*
  *   Jackbeat - Audio Sequencer
  *
- *   Copyright (c) 2004-2006 Olivier Guilyardi <olivier {at} samalyse {dot} com>
+ *   Copyright (c) 2004-2008 Olivier Guilyardi <olivier {at} samalyse {dot} com>
  *    
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  *   along with this program; if not, write to the Free Software
  *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- *   SVN:$Id: sequence.c 64 2007-02-28 17:27:46Z olivier $
+ *   SVN:$Id: sequence.c 126 2008-01-10 19:31:37Z olivier $
  */
 
 #define SEQUENCE_MASK_ATTACK_DELAY 3.0 // Miliseconds
@@ -135,7 +135,7 @@ struct sequence_t
   int               auto_connect;
   int               sr_converter_default_type;
   msg_t *           msg;
-
+  int               error;
 };
 
 /****************************************************************************
@@ -150,33 +150,32 @@ typedef struct sequence_msg_t
 
 #define SEQUENCE_MSG_SIZE 128
 
-#define SEQUENCE_MSG_SET_BPM 2
-#define SEQUENCE_MSG_SET_TRANSPORT 4
-#define SEQUENCE_MSG_SET_LOOPING 5
-#define SEQUENCE_MSG_UNSET_LOOPING 6
-#define SEQUENCE_MSG_START 7
-#define SEQUENCE_MSG_STOP 8
-#define SEQUENCE_MSG_REWIND 9
-#define SEQUENCE_MSG_SET_SR_RATIO 10
-#define SEQUENCE_MSG_SET_VOLUME 11
-#define SEQUENCE_MSG_SET_SMOOTHING 12
+#define SEQUENCE_MSG_SET_BPM        2
+#define SEQUENCE_MSG_SET_TRANSPORT  4
+#define SEQUENCE_MSG_SET_LOOPING    5
+#define SEQUENCE_MSG_UNSET_LOOPING  6
+#define SEQUENCE_MSG_START          7
+#define SEQUENCE_MSG_STOP           8
+#define SEQUENCE_MSG_REWIND         9
+#define SEQUENCE_MSG_SET_SR_RATIO   10
+#define SEQUENCE_MSG_SET_VOLUME     11
+#define SEQUENCE_MSG_SET_SMOOTHING  12
 
 #define SEQUENCE_MSG_NO_ACK -32
+#define SEQUENCE_MSG_ACK    33
 
-#define SEQUENCE_MSG_ACK 33
-
-#define SEQUENCE_MSG_LOCK_TRACKS 34
-#define SEQUENCE_MSG_UNLOCK_TRACKS 35
-#define SEQUENCE_MSG_LOCK_SINGLE_TRACK 36
-#define SEQUENCE_MSG_UNLOCK_SINGLE_TRACK 37
-#define SEQUENCE_MSG_RESIZE 38
-#define SEQUENCE_MSG_SET_SAMPLE 39
-#define SEQUENCE_MSG_MUTE_TRACK 40
-#define SEQUENCE_MSG_ENABLE_MASK 41
-#define SEQUENCE_MSG_DISABLE_MASK 42
-#define SEQUENCE_MSG_ACK_STOP 43
-#define SEQUENCE_MSG_ACK_ENABLE 44
-#define SEQUENCE_MSG_ACK_DISABLE 45
+#define SEQUENCE_MSG_LOCK_TRACKS          34
+#define SEQUENCE_MSG_UNLOCK_TRACKS        35
+#define SEQUENCE_MSG_LOCK_SINGLE_TRACK    36
+#define SEQUENCE_MSG_UNLOCK_SINGLE_TRACK  37
+#define SEQUENCE_MSG_RESIZE               38
+#define SEQUENCE_MSG_SET_SAMPLE           39
+#define SEQUENCE_MSG_MUTE_TRACK           40
+#define SEQUENCE_MSG_ENABLE_MASK          41
+#define SEQUENCE_MSG_DISABLE_MASK         42
+#define SEQUENCE_MSG_ACK_STOP             43
+#define SEQUENCE_MSG_ACK_ENABLE           44
+#define SEQUENCE_MSG_ACK_DISABLE          45
 
 
 /*******************************
@@ -434,7 +433,7 @@ sequence_receive_messages (sequence_t * sequence)
         break;
       case SEQUENCE_MSG_ACK_STOP:
         sequence->status = SEQUENCE_DISABLED;
-        if (sequence->transport_query) jack_transport_stop (sequence->client);
+        // Not stopping jack transport
         break;
       case SEQUENCE_MSG_ACK_ENABLE:
         sequence->status = SEQUENCE_ENABLED;
@@ -571,7 +570,6 @@ sequence_do_process (sequence_t *sequence, jack_nframes_t position, jack_nframes
                     {
                       *(t->frame_pos) = 0;
                       t->active_beat = current_beat;
-                      
                     }
                 }
              
@@ -604,7 +602,6 @@ sequence_do_process (sequence_t *sequence, jack_nframes_t position, jack_nframes
               if (dist != -1)
                {
                 offset_next = beat_nframes - current_offset + (dist - 1) * beat_nframes; 
-                
                }
                    
               if ((t->active_beat != -1) && (!t->beats[t->active_beat])) 
@@ -615,7 +612,8 @@ sequence_do_process (sequence_t *sequence, jack_nframes_t position, jack_nframes
                 if (t->sr_converter != NULL) src_reset (t->sr_converter);
                }
               
-              mask = (t->mask != NULL) ? t->mask[current_beat] : 1;
+              mask = ((current_beat < sequence->beats_num) && (t->mask != NULL)) 
+                      ? t->mask[current_beat] : 1;
 
               n = sequence_copy_sample_data (sequence, i, nframes - footer, 
                     mask & t->enabled, offset_next, &(t->current_level));
@@ -694,6 +692,7 @@ sequence_init (sequence_t * sequence)
   sequence->auto_connect = 1;
   sequence->msg = NULL;
   sequence->sr_converter_default_type = SEQUENCE_LINEAR;
+  sequence->error = 0;
 }
 
 static void
@@ -810,10 +809,11 @@ sequence_make_port_name (sequence_track_t *track, int channel)
   return s;
 }
 
-static void
+static int 
 sequence_register_track (sequence_t *sequence, sequence_track_t *track)
 {
   int j;
+  sequence->error = 0;
 
   DEBUG("Channels: %d", track->channels_num);
   for (j = 0; j < track->channels_num; j++) {
@@ -821,7 +821,26 @@ sequence_register_track (sequence_t *sequence, sequence_track_t *track)
     track->channels[j] = jack_port_register (sequence->client, name, JACK_DEFAULT_AUDIO_TYPE, 
                                              JackPortIsOutput, 0);
     free(name);
+    if (!track->channels[j]) {
+      DEBUG("Failed to register JACK port");
+      // Rolling back
+      for (j--; j >= 0; j--) 
+        jack_port_unregister(sequence->client, track->channels[j]);
+
+      sequence->error = ERR_SEQUENCE_JACK_REGPORT;
+      break;
+    }
   }
+  return sequence->error ? 0 : 1;
+}
+
+static void
+sequence_stop_ack (sequence_t * sequence)
+{
+  sequence_msg_t msg;
+  msg.type = SEQUENCE_MSG_ACK_STOP;
+  msg.text[0] = '\0';
+  msg_send (sequence->msg, &msg, MSG_ACK);
 }
 
 /*********************************
@@ -866,11 +885,23 @@ sequence_new (char *name, int *error)
   sequence->framerate = jack_get_sample_rate (sequence->client);
   DEBUG ("Jack Framerate : %d", sequence->framerate);
 
-  sequence_resize (sequence, 4, 16, 4, 0);
+  if (!sequence_resize (sequence, 4, 16, 4, 0)) 
+  {
+    DEBUG("Can't set initial sequence size");
+    *error = sequence->error;
+    sequence_destroy (sequence);
+    return NULL;
+  }
   
   return sequence;
 }
 
+int
+sequence_get_error (sequence_t * sequence)
+{
+  return sequence->error;
+}
+
 void
 sequence_rewind (sequence_t * sequence)
 {
@@ -905,6 +936,7 @@ sequence_destroy (sequence_t * sequence)
   sequence_track_t *t;
   if (sequence->client != NULL)
    {
+    sequence_stop_ack (sequence);
     jack_deactivate (sequence->client);
     jack_client_close (sequence->client);
    }
@@ -923,12 +955,13 @@ sequence_destroy (sequence_t * sequence)
   free (sequence);
 }
 
-void
+int
 sequence_resize (sequence_t * sequence, int tracks_num, int beats_num,
                  int measure_len, int duplicate_beats)
 {
   int i, j;
   sequence_msg_t msg;
+  sequence->error = 0;
   
   /* Asks the audio thread to avoid playing (lock) tracks whose jack ports are
      about to be unregistered */
@@ -973,8 +1006,21 @@ sequence_resize (sequence_t * sequence, int tracks_num, int beats_num,
       do sprintf (t->name, "track%d", j++);
       while (sequence_track_name_exists (sequence, t->name));
 
-      sequence_register_track (sequence, t);
-      if (sequence->auto_connect) sequence_connect_playback (sequence, t);
+      if (sequence_register_track (sequence, t)) {
+        if (sequence->auto_connect) sequence_connect_playback (sequence, t);
+      } else {
+        DEBUG("Couldn't not register track, stopping resize");
+        free(t->channels);
+        free(t->buffers);
+        free(t->frame_pos);
+        if ((tracks_num = i) > 0) {
+          new_tracks = realloc (new_tracks, tracks_num * sizeof(sequence_track_t));
+        } else {
+          free (new_tracks);
+          new_tracks = NULL;
+        }
+        break;
+      }
     }
 
   /* Resize beats memory */
@@ -1018,7 +1064,7 @@ sequence_resize (sequence_t * sequence, int tracks_num, int beats_num,
   sprintf (msg.text, "tracks=%p tracks_num=%d beats_num=%d measure_len=%d",
             new_tracks, tracks_num, beats_num, measure_len);
 
-  DEBUG("Sending resize message of length : %d",strlen(msg.text));
+  DEBUG("Sending resize message : %s", msg.text);
   sequence_track_t *old_tracks = sequence->tracks;
   int old_tracks_num = sequence->tracks_num;
 
@@ -1026,9 +1072,9 @@ sequence_resize (sequence_t * sequence, int tracks_num, int beats_num,
   msg_send (sequence->msg, &msg, MSG_ACK);
 
   /* Cleans up old data */
-  DEBUG("Cleaning up old tracks data at %p", old_tracks);
   if (old_tracks)
     {
+      DEBUG("Cleaning up old tracks data at %p", old_tracks);
       for (i = 0; i < old_tracks_num; i++)
        {
         free ((old_tracks + i)->beats);
@@ -1037,7 +1083,7 @@ sequence_resize (sequence_t * sequence, int tracks_num, int beats_num,
       free (old_tracks);
     }
   
-  DUMP(sequence->tracks->beats, sequence->beats_num);
+  return sequence->error ? 0 : 1; // sequence->error might get set by sequence_register_track()
 }
 
 void
@@ -1220,28 +1266,35 @@ sequence_get_sample (sequence_t * sequence, int track)
   return (sequence->tracks + track)->sample;
 }
 
-void
+int 
 sequence_set_sample (sequence_t * sequence, int track, sample_t * sample)
 {
   sequence_msg_t msg;
   sequence_track_t *t = sequence->tracks + track;
+  sequence_track_t old_track;
   int j, sr_converter_error;
+  sequence->error = 0;
   if (t->channels_num != sample->channels_num)
-    {
-      msg.type = SEQUENCE_MSG_LOCK_SINGLE_TRACK;
-      sprintf (msg.text, "track=%d", track);
-      msg_send (sequence->msg, &msg, MSG_ACK);
-      if (t->channels_num) {
-        for (j = 0; j < t->channels_num; j++)
-          jack_port_unregister (sequence->client, t->channels[j]);
-        free (t->channels);
-        free (t->buffers);
-      }
-      t->channels_num = sample->channels_num;
-      t->channels = calloc (t->channels_num, sizeof (jack_port_t *));
-      t->buffers = calloc (t->channels_num, sizeof (float *));
+   {
+    msg.type = SEQUENCE_MSG_LOCK_SINGLE_TRACK;
+    sprintf (msg.text, "track=%d", track);
+    msg_send (sequence->msg, &msg, MSG_ACK);
 
-      sequence_register_track (sequence, t);
+    memcpy(&old_track, t, sizeof (sequence_track_t));
+    t->buffers = calloc (t->channels_num, sizeof (float *));
+    t->channels_num = sample->channels_num;
+    t->channels = calloc (t->channels_num, sizeof (jack_port_t *));
+
+    if (sequence_register_track (sequence, t))
+     {
+
+      if (old_track.channels_num) 
+       {
+        for (j = 0; j < old_track.channels_num; j++)
+          jack_port_unregister (sequence->client, old_track.channels[j]);
+        free (old_track.channels);
+        free (old_track.buffers);
+       }
 
       if (t->sr_converter != NULL) 
        {
@@ -1250,7 +1303,21 @@ sequence_set_sample (sequence_t * sequence, int track, sample_t * sample)
         free (t->sr_converter_buffer);
         t->sr_converter_buffer = NULL;
        }
-    }
+     } 
+    else 
+     {
+      DEBUG("Couldn't register track");
+      free(t->channels);
+      free(t->buffers);
+      t->channels = old_track.channels;
+      t->channels_num = old_track.channels_num;
+      t->buffers = old_track.buffers;
+      msg.type = SEQUENCE_MSG_UNLOCK_SINGLE_TRACK;
+      sprintf (msg.text, "track=%d", track);
+      msg_send (sequence->msg, &msg, MSG_ACK);
+      return 0; // Failed, exiting..
+     }
+   }
   
   if (t->sr_converter_type == -1) 
     t->sr_converter_type = sequence->sr_converter_default_type;
@@ -1290,6 +1357,7 @@ sequence_set_sample (sequence_t * sequence, int track, sample_t * sample)
   sprintf (msg.text, "track=%d sample=%p", track, sample);
   msg_send (sequence->msg, &msg, MSG_ACK);
   if (sequence->auto_connect) sequence_connect_playback (sequence, t);
+  return 1;
 }
 
 void
@@ -1517,6 +1585,32 @@ sequence_get_resampler_type (sequence_t *sequence)
   return sequence->sr_converter_default_type;
 }
 
+static long int
+sequence_get_sustain_nframes (sequence_t *sequence)
+{
+  int i, j;
+  int beat_nframes = sequence_get_beat_nframes (sequence);
+  long int sustain = 0, track_sustain;
+
+  for (i = 0; i < sequence->tracks_num; i++)
+   {
+    sequence_track_t *track = sequence->tracks + i;
+    if (track->sample) 
+     {
+      long int sample_nframes = track->sample->frames * track->sr_converter_ratio;
+      for (j = sequence->beats_num; (j >= 0) && (!track->beats[j]) ; j--)
+        ;
+      if (j != -1)
+       {
+        track_sustain = sample_nframes - (sequence->beats_num - j) * beat_nframes;
+        if (track_sustain > sustain)
+          sustain = track_sustain;
+       }
+     }
+   }
+   return sustain;
+}
+
 void static 
 sequence_export_mix (sequence_t *sequence, float *output, int nframes)
 {
@@ -1540,7 +1634,7 @@ sequence_export_mix (sequence_t *sequence, float *output, int nframes)
 }
 
 void
-sequence_export (sequence_t *sequence, char *filename,  
+sequence_export (sequence_t *sequence, char *filename, int framerate, int sustain_type,
                  progress_callback_t progress_callback, void *progress_data)
 {
   jack_nframes_t bufsize;
@@ -1566,15 +1660,13 @@ sequence_export (sequence_t *sequence, char *filename,
     msg_send (sequence->msg, &msg, MSG_ACK);
    }
 
-  sequence_nframes = sequence->beats_num * sequence_get_beat_nframes (sequence);
-
   sequence_tmp = malloc (sizeof (sequence_t));
   memcpy (sequence_tmp, sequence, sizeof (sequence_t));
 
   sequence_tmp->tracks = calloc (sequence->tracks_num, sizeof (sequence_track_t));
   memcpy (sequence_tmp->tracks, sequence->tracks, sequence->tracks_num * sizeof (sequence_track_t));
 
-  sequence_tmp->looping = 0;
+  sequence_tmp->looping = (sustain_type == SEQUENCE_SUSTAIN_LOOP);
 
   // Allocating temporary buffers
   for (i = 0; i < sequence->tracks_num; i++) 
@@ -1583,56 +1675,107 @@ sequence_export (sequence_t *sequence, char *filename,
     track->buffers = calloc (track->channels_num, sizeof (float *));
     for (j = 0; j < track->channels_num; j++)
       track->buffers[j] = calloc (bufsize, sizeof(float));
-    *(track->frame_pos) = 0;
+    if (track->sample)
+      *(track->frame_pos) = track->sample->frames;
     track->active_beat = -1;
+    if (track->sample) 
+      track->sr_converter_ratio = track->sr_converter_ratio * framerate / sequence_tmp->framerate;
    }
 
+  sequence_tmp->framerate = framerate;
+  sequence_nframes = sequence->beats_num * sequence_get_beat_nframes (sequence_tmp);
+
   output = calloc (2 * bufsize, sizeof(float));
 
   // Opening file for writing
-  info.samplerate = sequence->framerate;
+  info.samplerate = sequence_tmp->framerate;
   info.channels = 2;
   info.format = SF_FORMAT_WAV | SF_FORMAT_PCM_16;
   fd = sf_open (filename, SFM_WRITE, &info); // Needs error checking
   if (fd) { DEBUG("Successfully opened outfile"); }
   else { DEBUG("FAILED to open outfile"); sf_perror(fd); }
 
+  long int process_total_nframes = 0, process_pos = 0;
+  switch (sustain_type)
+   {
+    case SEQUENCE_SUSTAIN_LOOP:
+      process_total_nframes = sequence_nframes * 4;
+      break;
+    case SEQUENCE_SUSTAIN_TRUNCATE:
+      process_total_nframes = sequence_nframes * 3;
+      break;
+    case SEQUENCE_SUSTAIN_KEEP:
+      process_total_nframes = (sequence_nframes + sequence_get_sustain_nframes (sequence)) * 3;
+      break;
+   } 
+
+  // Dry playing once for loop mixing
+  if (sustain_type == SEQUENCE_SUSTAIN_LOOP) 
+    for (pos = 0; pos < sequence_nframes; pos += bufsize) 
+     {
+      progress_callback ("Preparing loop", 0.1 + (float) process_pos / process_total_nframes * 0.9, progress_data);
+      nframes = pos + bufsize < sequence_nframes ? bufsize : sequence_nframes - pos;
+      sequence_do_process (sequence_tmp, pos, nframes);
+      process_pos += nframes;
+     }
+
   // Finding level peak
   float peak = 0;
   for (pos = 0; pos < sequence_nframes; pos += bufsize) 
    {
-    progress_callback ("Computing level peak", 0.1 + (float) pos / sequence_nframes * 0.3, progress_data);
+    progress_callback ("Computing level peak", 0.1 + (float) process_pos / process_total_nframes * 0.9, progress_data);
     nframes = pos + bufsize < sequence_nframes ? bufsize : sequence_nframes - pos;
     sequence_do_process (sequence_tmp, pos, nframes);
     sequence_export_mix (sequence_tmp, output, nframes);
     for (i = 0; i < (nframes * 2); i++) if (fabsf (output[i]) > peak) peak = fabsf(output[i]);
+    process_pos += nframes;
+   }
+ 
+  if (sustain_type == SEQUENCE_SUSTAIN_KEEP)
+   {
+    for (; (nframes_played = sequence_do_process (sequence_tmp, pos, bufsize)); pos += bufsize) 
+     {
+      progress_callback ("Computing level peak", 0.1 + (float) process_pos / process_total_nframes * 0.9, progress_data);
+      sequence_export_mix (sequence_tmp, output, nframes_played);
+      for (i = 0; i < (nframes_played * 2); i++) if (fabsf (output[i]) > peak) peak = fabsf(output[i]);
+      process_pos += nframes_played;
+     }
    }
 
-  for (; (nframes_played = sequence_do_process (sequence_tmp, pos, bufsize)); pos += bufsize) 
-   {
-    sequence_export_mix (sequence_tmp, output, nframes_played);
-    for (i = 0; i < (nframes_played * 2); i++) if (fabsf (output[i]) > peak) peak = fabsf(output[i]);
-   }
+  if (sustain_type != SEQUENCE_SUSTAIN_LOOP)
+    for (i = 0; i < sequence_tmp->tracks_num; i++) 
+     {
+      track = sequence_tmp->tracks + i;
+      if (track->sample)
+        *(track->frame_pos) = track->sample->frames;
+      track->active_beat = -1;
+     }
 
   DEBUG ("level peak is: %f", peak);
 
   // Writing to file
   for (pos = 0; pos < sequence_nframes; pos += bufsize) 
    {
-    progress_callback ("Mixing and writing to file", 0.4 + (float) pos / sequence_nframes * 0.6, progress_data);
+    progress_callback ("Mixing and writing to file", 0.1 + (float) process_pos / process_total_nframes * 0.9, progress_data);
     nframes = pos + bufsize < sequence_nframes ? bufsize : sequence_nframes - pos;
     sequence_do_process (sequence_tmp, pos, nframes);
     sequence_export_mix (sequence_tmp, output, nframes);
     for (i = 0; i < (nframes * 2); i++) output[i] /= peak;
     sf_writef_float (fd, output, nframes);
+    process_pos += nframes * 2;
    }
 
-  for (; (nframes_played = sequence_do_process (sequence_tmp, pos, bufsize)); pos += bufsize) 
-   {
-    sequence_export_mix (sequence_tmp, output, nframes_played);
-    for (i = 0; i < (nframes_played * 2); i++) output[i] /= peak;
-    sf_writef_float (fd, output, nframes_played);
-   }
+  if (sustain_type == SEQUENCE_SUSTAIN_KEEP)
+    for (; (nframes_played = sequence_do_process (sequence_tmp, pos, bufsize)); pos += bufsize) 
+     {
+      progress_callback ("Mixing and writing to file", 0.1 + (float) process_pos / process_total_nframes * 0.9, progress_data);
+      sequence_export_mix (sequence_tmp, output, nframes_played);
+      for (i = 0; i < (nframes_played * 2); i++) output[i] /= peak;
+      sf_writef_float (fd, output, nframes_played);
+      process_pos += nframes_played * 2;
+     }
+
+  DEBUG("Processed %ld frames on an estimated total of %ld", process_pos, process_total_nframes);
 
   sf_close (fd);
 
